@@ -1,12 +1,12 @@
 from django import forms
+from django.utils.translation import ugettext_lazy as _
 
-from floppyforms.widgets import (TextInput, HiddenInput, CheckboxInput, Select,
-                                 ClearableFileInput, SelectMultiple,
-                                 DateInput, DateTimeInput, TimeInput, URLInput,
-                                 NumberInput, EmailInput, NullBooleanSelect,
-                                 SlugInput, IPAddressInput,
-                                 SplitDateTimeWidget,
-                                 SplitHiddenDateTimeWidget)
+from .widgets import (TextInput, HiddenInput, CheckboxInput, Select,
+                      ClearableFileInput, SelectMultiple, DateInput,
+                      DateTimeInput, TimeInput, URLInput, NumberInput,
+                      EmailInput, NullBooleanSelect, SlugInput, IPAddressInput,
+                      SplitDateTimeWidget, SplitHiddenDateTimeWidget,
+                      FILE_INPUT_CONTRADICTION)
 
 __all__ = (
     'Field', 'CharField', 'IntegerField', 'DateField', 'TimeField',
@@ -32,6 +32,15 @@ class Field(forms.Field):
 class CharField(Field, forms.CharField):
     widget = TextInput
 
+    def widget_attrs(self, widget):
+        attrs = super(CharField, self).widget_attrs(widget)
+        if attrs is None:
+            attrs = {}
+        if self.max_length is not None and isinstance(widget, (TextInput, HiddenInput)):
+            # The HTML attribute is maxlength, not max_length.
+            attrs.update({'maxlength': str(self.max_length)})
+        return attrs
+
 
 class BooleanField(Field, forms.BooleanField):
     widget = CheckboxInput
@@ -55,6 +64,29 @@ class FilePathField(ChoiceField, forms.FilePathField):
 
 class FileField(Field, forms.FileField):
     widget = ClearableFileInput
+    default_error_messages = {
+        'invalid': _(u"No file was submitted. Check the encoding type on the form."),
+        'missing': _(u"No file was submitted."),
+        'empty': _(u"The submitted file is empty."),
+        'max_length': _(u'Ensure this filename has at most %(max)d characters (it has %(length)d).'),
+        'contradiction': _(u'Please either submit a file or check the clear checkbox, not both.')
+    }
+
+    def __init__(self, *args, **kwargs):
+        self.max_length = kwargs.pop('max_length', None)
+        self.allow_empty_file = kwargs.pop('allow_empty_file', False)
+        super(FileField, self).__init__(*args, **kwargs)
+
+    def clean(self, data, initial=None):
+        if data is FILE_INPUT_CONTRADICTION:
+            raise forms.ValidationError(self.error_messages['contradiction'])
+        if data is False:
+            if not self.required:
+                return False
+            data = None
+        if not data and initial:
+            return initial
+        return super(FileField, self).clean(data)
 
 
 class ImageField(Field, forms.ImageField):
@@ -65,18 +97,8 @@ class MultipleChoiceField(Field, forms.MultipleChoiceField):
     widget = SelectMultiple
 
 
-try:
-    Parent = forms.TypedMultipleChoiceField
-except AttributeError:  # Django < 1.3
-    class Parent(forms.MultipleChoiceField):
-        """No-op class for older Django versions"""
-        def __init__(self, *args, **kwargs):
-            kwargs.pop('coerce', None)
-            kwargs.pop('empty_value', None)
-            super(Parent, self).__init__(*args, **kwargs)
-
-
-class TypedMultipleChoiceField(MultipleChoiceField, Parent):
+class TypedMultipleChoiceField(MultipleChoiceField,
+                               forms.TypedMultipleChoiceField):
     pass
 
 
