@@ -7,6 +7,7 @@ try:
 except ImportError:
     """GDAL / GEOS not installed. Tests will fail if contrib.gis
     is installed, and will be skipped otherwise"""
+    GEOSGeometry = None  # noqa
 
 try:
     from django.utils import unittest as ut2
@@ -98,6 +99,11 @@ def skipUnlessInstalled(app):
     return _deferredSkip(condition, "%s is not installed" % app)
 
 
+def skipUnlessGisAvailable():
+    condition = lambda: GEOSGeometry is None
+    return _deferredSkip(condition, "GEOSGeometry could not be imported")
+
+
 class GisTests(TestCase):
     """Tests for the GeoDjango widgets"""
 
@@ -116,6 +122,7 @@ class GisTests(TestCase):
         self.assertTrue(wkt in rendered, rendered)
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_point(self):
         class PointForm(forms.Form):
             p = forms.gis.PointField()
@@ -143,6 +150,7 @@ class GisTests(TestCase):
             self.assertFalse(PointForm(data=data).is_valid())
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_multipoint(self):
         class PointForm(forms.Form):
             p = forms.gis.MultiPointField()
@@ -166,6 +174,7 @@ class GisTests(TestCase):
             self.assertFalse(PointForm(data=data).is_valid())
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_linestring(self):
         class LineStringForm(forms.Form):
             l = forms.gis.LineStringField()
@@ -189,6 +198,7 @@ class GisTests(TestCase):
             self.assertFalse(LineStringForm(data=data).is_valid())
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_multilinestring(self):
         class LineStringForm(forms.Form):
             l = forms.gis.MultiLineStringField()
@@ -212,6 +222,7 @@ class GisTests(TestCase):
             self.assertFalse(LineStringForm(data=data).is_valid())
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_polygon(self):
         class PolygonForm(forms.Form):
             p = forms.gis.PolygonField()
@@ -236,6 +247,7 @@ class GisTests(TestCase):
             self.assertFalse(PolygonForm(data=data).is_valid())
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_multipolygon(self):
         class PolygonForm(forms.Form):
             p = forms.gis.MultiPolygonField()
@@ -259,6 +271,7 @@ class GisTests(TestCase):
             self.assertFalse(PolygonForm(data=data).is_valid())
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_geometry(self):
         class GeometryForm(forms.Form):
             g = forms.gis.GeometryField()
@@ -283,6 +296,7 @@ class GisTests(TestCase):
             self.assertFalse(GeometryForm(data=data).is_valid())
 
     @skipUnlessInstalled('django.contrib.gis')
+    @skipUnlessGisAvailable()
     def test_geometrycollection(self):
         class GeometryForm(forms.Form):
             g = forms.gis.GeometryCollectionField()
